/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: Klaus Schneider,  VC


Program Description:
--------------------

The program is searching for black objects on a white background.
Black areas must be darker than the treshold value. The program
returns the size, the position of every black area and shows the
position on the monitor.

10.07.07 LB : removed wpix()

*/
/*******************************************************************/

#define NEW_IMAGE_VAR

#include <vcrt.h>
#include <vclib.h>
#include <macros.h>
#include <sysvar.h>

/*******************************************************************/

#ifndef NEW_IMAGE_VAR
 #define  EVEN(x)  ((x)&0xFFFE)
#else
 #define  EVEN(x)  (x)
#endif

/*******************************************************************/

void blobtest     (int x, int y, int dx, int dy, int color);
int  find_objects (image *area, ftr *f, int maxobjects, int th, int display);
void cross_image  (int line, int column, int size, int color);

/*******************************************************************/

int ms;

/*******************************************************************/

void main(void)

  {   
  int   x, y, dx, dy;
  char  resp;

  dx = EVEN(640);
  dy = EVEN(480);
  x  = EVEN((ScrGetColumns-dx)/2);
  y  = EVEN((ScrGetRows   -dy)/2);

    /*******************************************************************************/
    /*                 PLEASE INITIALIZE YOUR LICENCE CODES                        */
    /*                    To activate library functions                            */
    /*                                                                             */
    /* VCLIB :         init_licence("T1122334455") or init_licence("LT1122334455") */
    /* Extension Lib : init_licence("E1122334455") or init_licence("LE1122334455") */
    /* Color Lib :     init_licence("C1122334455") or init_licence("LC1122334455") */
    /* etc.                                                                        */
    /*                                                                             */
    /*      init_licence() returns 0 if licence code is OK, -5 otherwise           */
    /*                                                                             */
    /* Example :                                                                   */
    /*                                                                             */
	/**/ 	print("Init VCLIB = %d\n", init_licence("T1122334455"));			 /**/
    /*******************************************************************************/

  /* Working Page */
  ScrSetLogPage((int)ScrGetPhysPage);

  vmode(vmLive);
  print("\nblob analyse ('w'=white 'b'=black objects)\n"); resp=rs232rcv();
  tpict();
  
  do
    {
    tpict();

    if (resp=='w') blobtest(x, y, dx, dy, -1);
    if (resp=='b') blobtest(x, y, dx, dy,  0);

    print("\nblob analyse (%dx%d): t=%dms",dx,dy,ms); 
    print("\n\nnew run? ('w'=white 'b'=black 'q'=quit)");  
    resp=rs232rcv();
    }
  while(resp !='q' );

  /* live picture at the end */
  vmode(vmLive);
  }

/*******************************************************************/


/*******************************************************************/
/*********************** SUBROUTINE ********************************/
/*******************************************************************/


/*******************************************************************/

#define MAXOBJ  4000

void blobtest (int x, int y, int dx, int dy, int color)

  {
  int i, j=0, nobj,threshold=100,display=1;
  image area;
  ftr f[MAXOBJ];


  /* Search area is the whole image */
  ImageAssign(&area,ScrByteAddr(x,y),dx,dy,ScrGetPitch);
  
  /* feature extraction */
  nobj=find_objects(&area,f,MAXOBJ,threshold,display);
  
  print("\n\nFound %d white and black objects (-1 means more than %d objects)",nobj,MAXOBJ);

  if (color) print("\nwhite objects:");
  else       print("\nblack objects:");

  for(i=0;i<nobj;i++)
    {
    /* looking for color objects (black=0, white=-1) */
    if (f[i].color==color) 
      {
      /* print via seriell at 9600 Baud takes a long time */
      print("\nArea %d: Size=%d Center of Gravity = (%d,%d)",
               ++j,f[i].area,f[i].x_center+x,f[i].y_center+y);
      cross_image(EVEN((int)f[i].y_center+y),EVEN((int)f[i].x_center+x),10,127);
      }
    }
   }

/*******************************************************************/

int find_objects (image *a, ftr *f, int maxobjects, int th, int display)

  {
  long maxlng= 0x020000;
  U16 *rl1, *rl2, *slc;
  int  objects;

  /* allocate DRAM Memory */
  rl1 = rlcmalloc(maxlng);
  if (rl1 == NULL) {pstr("DRAM Memory overrun\n"); return(-1);}

  /* create RLC */
  ms = getvar(MSEC);
  rl2=rlcmk(a, th, rl1, maxlng);
  if (rl2 == NULL) {pstr("RLC overrun\n"); rlcfree(rl1); return(-1);}

  /* you can use some filter (dilate, erode) */
  if (1) /* 1=with filter 0=no filter */
    {
    slc=erode2(rl1,rl2);
    /* slc=dilate(rl1,rl2); */
    /* slc=rlc_mf(rl1,rl2,0,4); */
    }
  else
    {
    slc=rl2;
    rl2=rl1;
    }
  
 
  /* object labelling */
  if(sgmt(rl2,slc)==NULL) {pstr("object number overrun\n"); rlcfree(rl1); return(-1);}

  /* feature extraction */
  objects=rl_ftr2(rl2, f, maxobjects);

  ms = getvar(MSEC)-ms;
  if(ms<0) ms+=1000;
 
  /* display RLC */
  if (display)
   {
   if(rlcout(a,rl1,0,255)<0) printf("error in rlcout\n");
   }

  /* free allocation */
  rlcfree(rl1);  

  return(objects);
  }

/*******************************************************************/

void cross_image (int line, int column, int size, int color)

  {
  int i;


  for (i=-size;i<=size;i++)
    {
    *(ScrByteAddr(column  ,line+i)) = color;
	*(ScrByteAddr(column+i,line  )) = color;
    }
  }

/*******************************************************************/

/*******************************************************************/
/*******************************************************************/
/*******************************************************************/
