/*******************************************************************/
/*

Demo-Software for smart cameras from Vision Components
------------------------------------------------------

Program: MS,  VC


Program Description:
--------------------

This program shows how to write to / read from the serial device
of VC4XXX cameras.

*/
/*******************************************************************/

#include <register.h>
#include <vcrt.h>
#include <sysvar.h>

/*******************************************************************/

#define IO_IOCTL_SERIAL_INBUF            (0x0201) // serial only!
#define IO_IOCTL_SERIAL_OUTBUF           (0x0202)
#define IO_IOCTL_SERIAL_TRANSMIT         (0x0107)

#define BUFSIZE 25600

void main(void)
{
  FILE *tty;
  unsigned xbaud = 115200;
  char buf[BUFSIZE];
  char c = 0;
  int write1, write2, i, r;
  unsigned int result;

  INTERFACE_MODE(SERIAL);  /* Only for cameras with encoder interface (S/N xx1xxxx) */

  print("TEST VC40xx kbd: serial I/O\n");

  tty = fopen("kbd:", (void *)0);	/* Open serial device */


  print("Set baudrate=%d\n", xbaud);
  io_ioctl(tty, IO_IOCTL_SERIAL_SET_BAUD, &xbaud);     /* Set baud rate */
  xbaud=0;
  io_ioctl(tty, IO_IOCTL_SERIAL_SET_FLAGS, &xbaud);    /* No character interpretation */

  print("\nWriting 'Hello' to serial device !\n");
  write1 = write(tty, "Hello", 5);
  print("Characters sent : %d\n", write1);

  print("\nAny typed char on serial device will be echoed!\n");
  print("\nPress ESC on serial terminal to exit!\n");

  do
  {
	c = io_fgetc(tty);
	print("c=0x%x\n", c);

	if(c != 0x1b)
	  {
	    unsigned int result;
	    io_ioctl(tty, IO_IOCTL_SERIAL_OUTBUF, &result); // test if there are some chars in out buffer

	    if(result)
	    {
		  io_fputc(c, tty);
	    }
	  }

  }
  while(c != 0x1b);


  // Writing 'V' to serial device (BUFSIZE times)
  for(i=0;i<BUFSIZE;i++)
    buf[i] = 'V';
  write2 = write(tty, buf, BUFSIZE);

  // Writing 'C' to serial device (BUFSIZE times)
  for(i=0;i<BUFSIZE;i++)
    buf[i] = 'C';
  write2 = write(tty, buf, BUFSIZE);

  // Writing 5 times 'Hello again\n' to serial device
  for(i=0;i<5;i++)
  {
    print("\nWriting 'Hello again\n' to serial device !\n");
    write1 = write(tty, "Hello again\n", 12);
  }


  // Waiting for end of transmission before closing device (important not to lose characters)
  result = 0;
  do
  {
    io_ioctl(tty, IO_IOCTL_SERIAL_TRANSMIT, &result); 
    //print("res=%d chars in OUTBUF\n",result);
  }
  while(result);

  r = fclose(tty);	/* Close serial device : returns -1 if buffer wasn't sent entirely */
  print("Closing device = %d \n", r);

}



